class VarDoesNotExistException(Exception):
    pass

class UnsupportableDateFormatException(Exception):
    pass

class UnsupportableDateTimeFormatException(Exception):
    pass

class KeyNotFoundException(Exception):
    pass
